/*
   smartdongle.c: SmartDongle API v3.3


   MicroWorks, Inc.
   2808 North Cole Road
   Boise, ID 83704

   www.smartdongle.com


   ***** BEGIN LICENSE BLOCK *****

   Version: MPL 1.1/GPL 2.0/LGPL 2.1

   The contents of this file are subject to the Mozilla Public License
   Version 1.1 (the "License"); you may not use this file except in
   compliance with the License. You may obtain a copy of the License at
   http://www.mozilla.org/MPL/

   Software distributed under the License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
   for the specific language governing rights and limitations under the
   License.

   The Original Code is MicroWorks, Inc. code.

   The Initial Developer of the Original Code is MicroWorks, Inc. Portions
   created by MicroWorks, Inc. are Copyright (C) 2006
   MicroWorks, Inc. All Rights Reserved.

   Contributor(s):

   Alternatively, the contents of this file may be used under the terms of
   either the GNU General Public License Version 2 or later (the "GPL"), or
   the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
   in which case the provisions of the GPL or the LGPL are applicable instead
   of those above. If you wish to allow use of your version of this file only
   under the terms of either the GPL or the LGPL, and not to allow others to
   use your version of this file under the terms of the MPL, indicate your
   decision by deleting the provisions above and replace them with the notice
   and other provisions required by the GPL or the LGPL. If you do not delete
   the provisions above, a recipient may use your version of this file under
   the terms of any one of the MPL, the GPL or the LGPL.

   ***** END LICENSE BLOCK *****
*/
#include <windows.h>
#include <setupapi.h>
#include <stdlib.h>
#include <objbase.h>
#include "sd_user.h"
#include "sd_cy7c63743.h"
#include "sd_eeprom.h"
#include "sd_uskerr.h"
#include "smartdongle.h"
#include "sd_helper.h"

 #ifdef _MSC_VER
    /* 4996 warning to use MS extensions e.g., strcpy_s instead of strncpy */
    #pragma warning(disable: 4996)
 #endif

/*
   Given the correct keys, P1 and P2, will place the dongle into
   the enabled state and read EEPROM into buffer.
   Number of bytes read returned in size parameter.

   Note: Refer to uskerr.h for returned errors.
*/
int SmartDongleRead(
   unsigned __int64 P1,
   unsigned __int64 P2,
   unsigned int address,
   unsigned char *buffer,
   int size)
{
   int status;
   int bufferSize = size;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;


   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      return status;
   }

   /*
      Attempt to read user EEPROM space.
      On failure, attempt to disable dongle and return error.
      Number of bytes read returned in size parameter.
   */
   status = ReadEepromUserSpace(
      P1, P2, (unsigned short)address, buffer, &bufferSize, &m_UsbHandle);
   if (status != USK_OK)
   {
      DisableUsk(&m_UsbHandle);
      goto SD_ERROR;
   }

   /*
      Lock SmartDongle.
   */
   status = DisableUsk(&m_UsbHandle);

SD_ERROR:
   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   return status;
}


/*
   Given the correct keys, P1 and P2, will place the dongle into
   the enabled state and write buffer into EEPROM.
   Number of bytes written returned in size parameter.

   See sd_uskerr.h for returned errors.
*/
int SmartDongleWrite(
   unsigned __int64 P1,
   unsigned __int64 P2,
   unsigned int address,
   unsigned char *buffer,
   int size)
{
   int status;
   int bufferSize = size;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;


   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      return status;
   }

   /*
      Attempt to write to user EEPROM space.
      On failure, attempt to disable dongle and return error.
      Number of bytes written returned in size parameter.
   */
   status = WriteEepromUserSpace(
      P1, P2, (unsigned short)address, buffer, &bufferSize, &m_UsbHandle);
   if (status != USK_OK)
   {
      DisableUsk(&m_UsbHandle);
      goto SD_ERROR;
   }

   /*
      Lock SmartDongle.
   */
   status = DisableUsk(&m_UsbHandle);

SD_ERROR:
   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   return status;
}


/*
   Switch SmartDongle LED indicator to Green.
*/
int SmartDongleLedGreen(void)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
       return status;
   }

   status = Green(&m_UsbHandle);

   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   return status;
}


/*
   Switch SmartDongle LED indicator to Red.
*/
int SmartDongleLedRed(void)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      return status;
   }

   status = Red(&m_UsbHandle);

   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   return status;
}


/*
   Switch SmartDongle LED indicator to Off.
*/
int SmartDongleLedOff(void)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      return status;
   }

   status = Off(&m_UsbHandle);

   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   return status;
}


/*
   SmartDongle software reset;
*/
int SmartDongleReset(void)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;
   int i;


   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      return status;
   }

   SoftReset(&m_UsbHandle);

   Sleep(QTS_PLUG_N_PLAY_WAIT);

   for (i=0; i<QTS_READY_RETRIES; i++)
   {
      status = OpenUsbDev(&m_UsbHandle);
      if (status != USK_OK)
      {
         continue;
      }

      status = ready(0, &m_UsbHandle);
      if (status == 0)
      {
         break;
      }

      CloseHandle(m_UsbHandle);
      m_UsbHandle = INVALID_HANDLE_VALUE;

      Sleep(QTS_READY_WAIT);
   }

   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   if (status)
   {
      return USK_RESET_COMEREADY;
   }

   return USK_OK;
}


/*
   For SmartDongle version >=1.42

   Set the current user memory space mode.
   The mode will reset to default mode on reset.

   modes
   0 - User EPROM memory space protected by strong (AES) encryption
   1 - User EPROM memory space is unencrypted
   2 - User EPROM memory space protected by fast encryption
*/
int SmartDongleWriteCryptMode(
   unsigned __int64 P1,
   unsigned __int64 P2,
   unsigned char mode)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;


   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      return status;
   }

   status = SetMode(P1, P2, mode, &m_UsbHandle);

   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   return status;
}


/*
   For SmartDongle version >=1.42

   Set the current user memory space encryption mode.
*/
int SmartDongleReadCryptMode(unsigned char *mode)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      return status;
   }

   status = GetMode(mode, &m_UsbHandle);

   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   return status;
}


/*
   Get SmartDongle serial number
   User must allocate CP2_SERIALNUM_LENGTH+1 for returned serial number string.
*/
int SmartDongleGetSerialNumber(char *sn)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;

   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
      return status;
   }

   status = getSerialNumber(sn, &m_UsbHandle);

   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   return status;
}


unsigned int SmartDongleGetEpromSize(
   unsigned __int64 P1,
   unsigned __int64 P2)
{
   int status;


   /*
      SmartDongle has 32K EPROM?
   */
   status = SmartDongleRead(P1, P2, 0x7EAF, NULL, 0);
   if (status == USK_OK)
   {
      return 0x7EB0;
   }

   /*
      SmartDongle has 8K EPROM?
   */
   status = SmartDongleRead(P1, P2, 0x1EAF, NULL, 0);
   if (status == USK_OK)
   {
      return  0x1EB0;
   }

   /*
      Unknown size or read error.
   */
   return 0;
}


/*
   For SmartDongle version >=1.50
   SmartDongle will require driver >=2.3

   Warning: Once strict mode is set, SmartDongle
            cannot be reset to "non-strict mode"
*/
int SmartDongleSetStrict(void)
{
   int status;
   HANDLE m_UsbHandle = INVALID_HANDLE_VALUE;


   status = OpenUsbDev(&m_UsbHandle);
   if (status != USK_OK)
   {
       return status;
   }

   status = setStrictOnly(&m_UsbHandle);

   if (m_UsbHandle != INVALID_HANDLE_VALUE)
   {
      CloseHandle(m_UsbHandle);
   }

   return status;
}


/*
   Return error string corresponding to error code.
   User must allocate string size of QTS_ERROR_STRING_LENGTH_MAX
*/
void SmartDongleGetErrorString(char *errorString, int error)
{
   switch (error)
   {
      case USK_OK:
         strcpy(errorString, "no errors");
         break;

      case USK_ERR_OPEN:
         strcpy(errorString, "Error: Can't open SmartDongle.");
         break;

      case USK_ERR_IS_OPEN:
         strcpy(errorString, "Error: SmartDongle is already open.");
         break;

      case USK_ERR_MEM:
          strcpy(errorString, "Error: Memory allocation error.");

      case USK_ERR_EEPROM_SIZE:
          strcpy(errorString, "Error: Address offset or buffer exceeds SmartDongle memory size.");
         break;

      case USK_ERR_5:
         strcpy(errorString, "Error deprecated or reserved, this is an incorrect error value");
         break;

      case USK_ERR_SECURITY_STATE_TIMEOUT:
         strcpy(errorString, "Error: Timeout waiting for security state change.");
         break;

      case USK_ERR_KEY_STATE_TIMEOUT:
         strcpy(errorString, "Error: Timeout waiting for key state change");
         break;

      case USK_ERR_TWC_RESET_TIMEOUT:
         strcpy(errorString, "Error: Timeout waiting for TWC to reset");
         break;

      case USK_ERR_BUSY_TIMEOUT:
         strcpy(errorString, "Error: Timeout waiting for job in firmware task loop to finish.");
         break;

      case USK_ERR_NOT_ENABLED:
         strcpy(errorString, "Error: Illegal command, SmartDongle needs to be unlocked first.");
         break;

      case USK_ERR_EEPROM_READ_TIMEOUT:
         strcpy(errorString, "Error: Timeout waiting for SmartDongle memory read.");
         break;

      case USK_ERR_EEPROM_READ:
         strcpy(errorString, "Error: SmartDongle memory Read error.");
         break;

      case USK_ERR_EEPROM_WRITE:
         strcpy(errorString, "Error: SmartDongle memory Write error.");
         break;

      case USK_ERR_USK_VALIDATE:
         strcpy(errorString, "Error deprecated or reserved, this is an incorrect error value");
         break;

      case USK_ERR_HOST_VALIDATE:
         strcpy(errorString, "Error: Bad keys, could not unlock SmartDongle");
         break;

      case USK_ERR_DISABLE:
         strcpy(errorString, "Error: Unable to lock SmartDongle.");
         break;

      case USK_ERR_DATA_FORMAT:
         strcpy(errorString, "Error: Format of Serial Number read from SmartDongle is bad");
         break;

     /*
        IOCTL Errors
     */
      case USK_ERR_IOCTL_GET_PIPE_INFO:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_PIPE_INFO");
         break;

      case USK_ERR_IOCTL_GET_DEVICE_DESCRIPTOR:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_DEVICE_DESCRIPTOR");
         break;

      case USK_ERR_IOCTL_GET_CONFIG_DESCRIPTOR:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_CONFIG_DESCRIPTOR");
         break;

      case USK_ERR_IOCTL_REGISTER_NOTIFY_EVENT:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_REGISTER_NOTIFY_EVENT");
         break;

      case USK_ERR_IOCTL_RESET_DEVICE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_RESET_DEVICE");
         break;

      case USK_ERR_IOCTL_RESET_PIPE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_RESET_PIPE");
         break;

      case USK_ERR_IOCTL_GET_DRIVER_REVISION:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_DRIVER_REVISION");
         break;

      case USK_ERR_IOCTL_GET_STRING_DESCRIPTOR:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_STRING_DESCRIPTOR");
         break;

      case USK_ERR_26:
         strcpy(errorString, "Error deprecated or reserved, this is an incorrect error value");
         break;

      case USK_ERR_IOCTL_SEND_KEY:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_SEND_KEY");
         break;

      case USK_ERR_IOCTL_GET_KEY:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_KEY");
         break;

      case USK_ERR_IOCTL_WRITE_PORT:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_WRITE_PORT");
         break;

      case USK_ERR_IOCTL_READ_PORT:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_READ_PORT");
         break;

      case USK_ERR_IOCTL_READ_EEPROM:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_READ_EEPROM");
         break;

      case USK_ERR_IOCTL_WRITE_EEPROM:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_WRITE_EEPROM");
         break;

      case USK_ERR_IOCTL_GET_SECURITY_STATE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_SECURITY_STATE");
         break;

      case USK_ERR_IOCTL_RESET_SECURITY_STATE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_RESET_SECURITY_STATE");
         break;

      case USK_ERR_IOCTL_GET_KEY_STATE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_KEY_STATE");
         break;

      case USK_ERR_IOCTL_RESET_KEY_STATE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_RESET_KEY_STATE");
         break;

      case USK_ERR_IOCTL_SEND_HOST_VERIFY_REQUEST:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_SEND_HOST_VERIFY_REQUEST");
         break;

      case USK_ERR_IOCTL_REQUEST_PAGE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_REQUEST_PAGE");
         break;

      case USK_ERR_IOCTL_GET_PAGE_FLAGS:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_PAGE_FLAGS");
         break;

      case USK_ERR_IOCTL_GET_TWC_FLAG:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_TWC_FLAG");
         break;

      case USK_ERR_IOCTL_GET_BUSY_FLAG:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_BUSY_FLAG");
         break;

      case USK_ERR_IOCTL_GET_EEPROM_ERROR_FLAG:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_EEPROM_ERROR_FLAG");
         break;

      case USK_ERR_43:
         strcpy(errorString, "Error deprecated or reserved, this is an incorrect error value");
         break;

      case USK_ERR_44:
         strcpy(errorString, "Error deprecated or reserved, this is an incorrect error value");
         break;

      case USK_ERR_IOCTL_QTSDONGLE_SEND_PORT:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_QTSDONGLE_SEND_PORT");
         break;

      case USK_ERR_IOCTL_QTSDONGLE_GET_FLAGS:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_QTSDONGLE_GET_FLAGS");
         break;

      case USK_ERR_IOCTL_SET_MODE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_SET_MODE");
         break;

      case USK_ERR_IOCTL_GET_MODE:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_GET_MODE");
         break;

      /*
         No device found using given keys or serial number
      */
      case USK_ERR_NODEV:
         strcpy(errorString, "Error: No SmartDongles found.");
         break;


      case USK_ERR_BAD_ARGUMENTS:
          strcpy(errorString, "Error: One or more function arguments are incorrect.");
         break;


      /*
         For SmartDongle >=1.50
      */
      case USK_ERR_80:
         strcpy(errorString, "Error deprecated or reserved, this is an incorrect error value");
         break;

      case USK_ERR_IOCTL_QTSDONGLE_SET_STRICT:
         strcpy(errorString, "Error: IOCTL failed: IOCTL_QTSDONGLE_SET_STRICT");
         break;

      case USK_ERR_REQUIRES_DRIVER_2_3_0:
         strcpy(errorString, "Error: An attempt to use 2.3 driver functionality with an older driver");
         break;

      case USK_ERR_REQUIRES_DONGLE_1_50:
         strcpy(errorString, "Error: An attempt to use new 1.50 SmartDongle functionality with an older dongle.");
         break;

      case USK_WARN_MODE_PAGE_BOUNDARY:
         strcpy(errorString, "Warning: Data written to user memory after encryption mode change was not on a page boundary.");
         break;

      case USK_WARN_CRYPT_PAGE_BOUNDARY:
         strcpy(errorString, "Warning: Encrypted data written to user memory was not on a page boundary.");
         break;

      case USK_RESET_COMEREADY:
         strcpy(errorString, "Error: SmartDongle did not come ready after a reset.");
         break;

      /*
         Unknown.
      */
      case USK_ERR_UNKNOWN:
         strcpy(errorString, "An unknown error occurred");
         break;

      default:
         strcpy (errorString, "An unknown error occurred");
         break;
   }
}
